<?php

/**
 * @file
 * Administration page callbacks for the metatag_taxonomy_facets module.
 */

/**
 * Form for selecting facets to use in combinations.
 *
 * @param array $form
 *   Form array.
 * @param array $form_state
 *   Form state array.
 *
 * @return array
 *   Created metatag_taxonomy_facets_configure_facets_form.
 */
function metatag_taxonomy_facets_configure_facets_form(array $form, array &$form_state) {
  $facets_selected = variable_get_value('metatag_taxonomy_facets_selected');

  $searchers = facetapi_get_searcher_info();
  foreach ($searchers as $searcher_name => $searcher_info) {
    $fieldset = array();
    $facets = facetapi_get_facet_info($searcher_name);
    foreach ($facets as $facet_name => $facet_info) {
      if ($facet_info['field type'] == 'taxonomy_term') {
        $facet_title = str_replace('Filter by ', '', $facet_info['description']);
        $fieldset["{$searcher_info['name']}:{$facet_name}"] = array(
          '#type' => 'checkbox',
          '#title' => $facet_title,
          '#default_value' => isset($facets_selected[$searcher_info['name']][$facet_name]),
        );
      }
    }

    if (!empty($fieldset)) {
      $fieldset += array(
        '#type' => 'fieldset',
        '#title' => $searcher_info['label'],
      );
      $form[$searcher_info['name']] = $fieldset;
    }
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Selecting facets form submit function.
 *
 * @param array $form
 *   Form being submitted.
 * @param array $form_state
 *   Form state being submitted.
 */
function metatag_taxonomy_facets_configure_facets_form_submit(array $form, array &$form_state) {
  $values = [];
  foreach ($form_state['values'] as $key => $value) {
    $key_exploded = explode(':', $key);
    if (count($key_exploded) > 1 && $value) {
      $searcher_name = array_shift($key_exploded);
      $values[$searcher_name][implode(':', $key_exploded)] = 1;
    }
  }

  variable_set_value('metatag_taxonomy_facets_selected', $values);

  $searchers = facetapi_get_searcher_info();
  $new_combinations = array();
  foreach ($searchers as $searcher_name => $searcher_info) {
    $facets = facetapi_get_facet_info($searcher_name);
    $selected_facets = [];
    foreach ($facets as $facet_name => $facet_info) {
      if ($facet_info['field type'] == 'taxonomy_term' && isset($values[$searcher_name][$facet_name])) {
        $selected_facets[$facet_name] = $facet_info;
      }
    }
    if (empty($selected_facets)) {
      continue;
    }
    $combinations = metatag_taxonomy_facets_get_combinations_info($searcher_info, $selected_facets);
    foreach ($combinations as $name => $label) {
      $new_combinations[$searcher_name][] = array(
        'searcher' => $searcher_name,
        'facets' => metatag_taxonomy_facets_get_facets_part($name, $searcher_name),
      );
    }
  }

  metatag_taxonomy_facets_save_combinations($new_combinations);

  metatag_config_cache_clear();
  token_clear_cache();
}

/**
 * Saves combinations: creates new, deletes stale.
 *
 * @param array $combinations
 *   Combinations being saved.
 */
function metatag_taxonomy_facets_save_combinations(array $combinations) {
  foreach ($combinations as $searcher_name => $searcher_combinations) {
    $prev_combinations = db_select('taxonomy_facets_combinations', 'c')
      ->fields('c', ['cid', 'facets'])
      ->condition('searcher', $searcher_name)
      ->execute()
      ->fetchAllKeyed();

    // Adding any new combinations.
    foreach ($searcher_combinations as $combination) {
      $prev_index = array_search($combination['facets'], $prev_combinations);
      if ($prev_index !== FALSE) {
        unset($prev_combinations[$prev_index]);
      }
      else {
        $entity = entity_create('taxonomy_facets_combinations', $combination);
        entity_save('taxonomy_facets_combinations', $entity);
      }
    }
    // Removing any stale combinations and corresponding entities.
    foreach ($prev_combinations as $cid => $facets) {
      metatag_taxonomy_facets_delete_combination($cid);
    }
  }
}

/**
 * Delete combination and corresponding instances records.
 *
 * @param int $cid
 *   Combintaion id to be deleted.
 */
function metatag_taxonomy_facets_delete_combination($cid) {
  $tfids = db_select('taxonomy_facets', 'f')
    ->fields('f', ['tfid'])
    ->condition('combination_id', $cid)
    ->execute()->fetchColumn();
  if (!empty($tfids)) {
    entity_delete_multiple('taxonomy_facets', $tfids);
  }

  db_delete('taxonomy_facets_combinations')
    ->condition('cid', $cid)
    ->execute();
}

/**
 * Facets combination instance metatags edit form.
 *
 * @param array $form
 *   Form array.
 * @param array $form_state
 *   Form state array.
 * @param object $taxonomy_facet
 *   Taxonomy facets entity being edited.
 *
 * @return array
 *   Created form.
 */
function metatag_taxonomy_facets_instance_edit_form(array $form, array &$form_state, $taxonomy_facet) {
  $form['tfid'] = array(
    '#type' => 'hidden',
    '#value' => $taxonomy_facet->tfid,
  );

  field_attach_form('taxonomy_facets', $taxonomy_facet, $form, $form_state, language_default());

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['actions']['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#submit' => array('metatag_taxonomy_facets_instance_edit_form_delete_submit'),
  );

  return $form;
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function metatag_taxonomy_facets_form_metatag_taxonomy_facets_instance_edit_form_alter(array &$form, array &$form_state) {
  $form['metatags']['#collapsible'] = FALSE;
  $form['metatags']['#collapsed'] = FALSE;
  $form['actions']['#weight'] = 100;
}

/**
 * Facets combination instance metatags edit form submit.
 *
 * @param array $form
 *   Form being submitted.
 * @param array $form_state
 *   Form state being submitted.
 */
function metatag_taxonomy_facets_instance_edit_form_submit(array $form, array &$form_state) {
  $entity = entity_load_single('taxonomy_facets', $form_state['values']['tfid']);
  $entity->metatags = $form_state['values']['metatags'];
  $result = entity_save('taxonomy_facets', $entity);
  if ($result === FALSE) {
    drupal_set_message(t('Error occurred while saving metatags for this instance'), 'error');
  }
}

/**
 * Facets combination instance metatags deleting submit.
 *
 * @param array $form
 *   Form being submitted.
 * @param array $form_state
 *   Form state being submitted.
 */
function metatag_taxonomy_facets_instance_edit_form_delete_submit(array $form, array &$form_state) {
  $result = entity_delete('taxonomy_facets', $form_state['values']['tfid']);
  if ($result === FALSE) {
    drupal_set_message(t('Error occurred while deleting metatags for this instance'), 'error');
  }
  else {
    drupal_set_message(t('Metatags for this instance successfully removed'));
    drupal_goto('admin/config/search/metatags');
  }
}

/**
 * Facets combination instance metatags add form.
 *
 * @param array $form
 *   Form array.
 * @param array $form_state
 *   Form state array.
 * @param int $combination_id
 *   Combination entity id.
 * @param string $tids
 *   String of taxonomy terms ids delimited by commas.
 *
 * @return array
 *   Created form.
 */
function metatag_taxonomy_facets_instance_add_form(array $form, array &$form_state, $combination_id, $tids) {
  $form['combination_id'] = [
    '#type' => 'hidden',
    '#value' => $combination_id,
  ];
  $form['tids'] = [
    '#type' => 'hidden',
    '#value' => urldecode($tids),
  ];

  $instance = metatag_taxonomy_facets_get_combination_instance($combination_id);
  $metatags = metatag_config_load($instance);
  $taxonomy_facet = entity_create('taxonomy_facets', array(
    'combination_id' => $combination_id,
    'tids' => $tids,
    'metatags' => $metatags->config,
  ));

  metatag_field_attach_form('taxonomy_facets', $taxonomy_facet, $form, $form_state, language_default());

  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Save'),
  ];

  return $form;
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function metatag_taxonomy_facets_form_metatag_taxonomy_facets_instance_add_form_alter(array &$form, array &$form_state) {
  $form['metatags']['#collapsible'] = FALSE;
  $form['metatags']['#collapsed'] = FALSE;
  $form['actions']['#weight'] = 100;
}

/**
 * Facets combination instance metatags add form submit.
 *
 * @param array $form
 *   Form being submitted.
 * @param array $form_state
 *   Form state being submitted.
 */
function metatag_taxonomy_facets_instance_add_form_submit(array $form, array &$form_state) {
  $values = array(
    'combination_id' => $form_state['values']['combination_id'],
    'tids' => $form_state['values']['tids'],
    'metatags' => $form_state['values']['metatags'],
  );
  $entity = entity_create('taxonomy_facets', $values);
  $result = entity_save('taxonomy_facets', $entity);

  if ($entity && $result !== FALSE) {
    $form_state['redirect'] = "admin/config/search/metatags/taxonomy_facets_item/{$entity->tfid}/edit";
  }
  else {
    drupal_set_message(t('Error occurred while saving metatags for this instance'), 'error');
  }
}
